"use strict";

// Long event name to avoid conflicts.
var _, baseOptions, browserify, _factory, glob, globOptions, gulp, updateSafe;

updateSafe = 'gulp-browserify-watchify-glob:bundle-done';

baseOptions = function baseOptions() {
  return {
    cache: {},
    packageCache: {}
  };
};

globOptions = {
  absolute: true
};

_factory = function factory(gulp, browserify, glob, _) {
  var accumulateEntriesSync, difference, _expandGlobs, expandGlobsSync, globbedBrowserify, updateEntries;

  accumulateEntriesSync = function accumulateEntriesSync(accumulator, pattern) {
    return accumulator.concat(glob.sync(pattern, globOptions));
  };

  expandGlobsSync = function expandGlobsSync(patterns) {
    return _.reduce(patterns, accumulateEntriesSync, []);
  };

  _expandGlobs = function expandGlobs(patterns, done, accumulator = []) {
    var pattern;
    pattern = _.head(patterns);

    if (!pattern) {
      return process.nextTick(done, null, accumulator);
    }

    return glob(pattern, globOptions, function (error, paths) {
      if (error) {
        return done(error);
      }

      return _expandGlobs(_.tail(patterns), done, accumulator.concat(paths));
    });
  };

  difference = function difference(left, right) {
    var diff;
    diff = new Set();
    left.forEach(function (item) {
      return right.has(item) || diff.add(item);
    });
    return diff;
  };

  updateEntries = function updateEntries(entriesGlob, entries, bundler, done) {
    return _expandGlobs(entriesGlob, function (error, paths) {
      var added, newEntries, oldEntries, pipeline, recorded, removed;

      if (error) {
        return done(error);
      } // The following logic relies on browserify implementation details.


      oldEntries = new Set(entries);
      newEntries = new Set(paths);
      added = difference(newEntries, oldEntries);
      removed = difference(oldEntries, newEntries);
      recorded = bundler._recorded;
      bundler.reset();
      pipeline = bundler.pipeline;
      added.forEach(function (path) {
        return bundler.add(path);
      });
      recorded.forEach(function (row) {
        if (removed.has(row.file)) {
          return;
        }

        if (row.entry) {
          row.order = bundler._entryOrder++;
        }

        return pipeline.write(row);
      });
      entries.splice(0);
      entries.splice(0, 0, ...paths);
      return done(error, entries);
    });
  };

  globbedBrowserify = function globbedBrowserify(options) {
    var afterBundling, beforeBundling, bundler, bundling, emitUpdate, entries, entriesChanged, entriesGlob, triggerBundle, watch, wrapTask;
    entriesGlob = options.entries;

    if (!entriesGlob) {
      throw TypeError('globbedBrowserify needs entries');
    }

    if (!_.isArray(entriesGlob)) {
      entriesGlob = [entriesGlob];
    }

    entries = expandGlobsSync(entriesGlob);
    bundler = new browserify(_.defaults({
      entries
    }, options, baseOptions()));
    bundling = false;
    entriesChanged = false;

    beforeBundling = function beforeBundling(done) {
      bundling = true;

      if (entriesChanged) {
        entriesChanged = false;
        return updateEntries(entriesGlob, entries, bundler, done);
      } else {
        return process.nextTick(done);
      }
    };

    beforeBundling.displayName = 'beforeBundling';

    afterBundling = function afterBundling(done) {
      bundling = false;
      bundler.emit(updateSafe);
      return process.nextTick(done);
    };

    afterBundling.displayName = 'afterBundling';

    triggerBundle = function triggerBundle(done) {
      entriesChanged = true;

      if (bundling) {
        return bundler.once(updateSafe, function () {
          return emitUpdate(done);
        });
      } else {
        return emitUpdate(done);
      }
    };

    emitUpdate = function emitUpdate(done) {
      bundler.emit('update');
      return process.nextTick(done);
    };

    wrapTask = function wrapTask(task) {
      return gulp.series(beforeBundling, task, afterBundling);
    };

    watch = function watch(task, kickoffTask = task) {
      var _closeWatcher, originalClose, updater, watchOptions, watcher;

      updater = _.debounce(wrapTask(task), 100);
      bundler.plugin('watchify');
      bundler.on('update', updater);
      watchOptions = {
        events: ['add', 'unlink'],
        cwd: process.cwd()
      };
      watcher = gulp.watch(entriesGlob, watchOptions, triggerBundle); // Function that must be invoked twice to take effect. See
      // github:paulmillr/chokidar/issues/612#issuecomment-378026034

      _closeWatcher = function closeWatcher() {
        return _closeWatcher = function closeWatcher() {
          return watcher.close();
        };
      };

      watcher.once('ready', function () {
        return _closeWatcher();
      });
      originalClose = bundler.close;

      bundler.close = function () {
        _closeWatcher();

        originalClose.call(bundler);
        bundler.removeListener('update', updater);
        return bundler.close = originalClose;
      };

      return wrapTask(kickoffTask);
    };

    bundler.watch = watch;
    bundler.wrap = wrapTask;
    return bundler;
  };

  return _.extend(globbedBrowserify, {
    default: globbedBrowserify,
    expandGlobsSync,
    expandGlobs: _expandGlobs,
    difference,
    updateEntries,
    factory: _factory
  });
};

gulp = require('gulp');
browserify = require('browserify');
glob = require('glob');
_ = require('underscore');
module.exports = _factory(gulp, browserify, glob, _);